@@ -1,8 +1,3 @@
-import string, os
-# @@ 2000-04-06 ce: Note that when causing an error by removing the above line,
-# the first part of the CGI output still gets through the error handling mechanism
-# of the CGI Wrapper
-
 
 print '''
 <html>
@@ -17,7 +12,7 @@ print '''
 def printKeys(name, obj):
 	keys = obj.keys()
 	keys.sort()
-	print '<p> <b>%s</b> = %s' % (name, string.join(keys, ', '))
+	print '<p> <b>%s</b> = %s' % (name, ', '.join(keys))
 
 printKeys('globals', globals())
 printKeys('locals', locals())
