@@ -87,11 +87,12 @@ Whitespace around field values is stripped.
 You can control all this behavior through the arguments found in the
 initializer and the various readFoo() methods:
 
-	...delimiter=',', allowComments=1, stripWhite=1
+	...delimiter=',', allowComments=True, stripWhite=True
 
 For example:
 
-	table = DataTable('foo.tabbed', delimiter='\t', allowComments=0, stripWhite=0)
+	table = DataTable('foo.tabbed', delimiter='\t',
+		allowComments=False, stripWhite=False)
 
 You should access these parameters by their name since additional ones
 could appear in the future, thereby changing the order.
@@ -165,9 +166,9 @@ CACHING
 DataTable uses "pickle caching" so that it can read .csv files faster
 on subsequent loads. You can disable this across the board with:
 	from MiscUtils.DataTable import DataTable
-	DataTable.usePickleCache = 0
+	DataTable._usePickleCache = False
 
-Or per instance by passing "usePickleCache=0" to the constructor.
+Or per instance by passing "usePickleCache=False" to the constructor.
 
 See the docstring of PickleCache.py for more information.
 
@@ -199,33 +200,19 @@ TO DO
 """
 
 
-import os, string, sys
-from CSVParser import CSVParser
-from CSVJoiner import joinCSVFields
-from string import join, replace, split, strip
+import os, sys
 from types import *
 
-try: # backward compatibility for Python < 2.2
-	StringTypes
-except NameError:
-	StringTypes = StringType
+from CSVParser import CSVParser
+from CSVJoiner import joinCSVFields
+from Funcs import positive_id
+from MiscUtils import NoDefault, StringTypes, mxDateTime
 
-try: # backward compatibility for Python < 2.3
+try: # for Python < 2.3
 	True, False
 except NameError:
 	True, False = 1, 0
 
-try:
-	from MiscUtils import NoDefault
-except ImportError:
-	class NoDefault:
-		pass
-
-try:
-	from mx.DateTime import DateTimeType, DateTimeFrom
-except ImportError:
-	pass
-
 
 ## Types ##
 
@@ -233,15 +220,15 @@ DateTimeType = "<custom-type 'datetime'>"
 ObjectType = "<type 'Object'>"
 
 _types = {
-	'string':	StringType,
-	'int':		IntType,
-	'bool':		IntType,
-	'long':		LongType,
-	'decimal':	FloatType,
-	'float':	FloatType,
-	'datetime':	DateTimeType,
-	'date' : DateTimeType,
-	'object':	ObjectType
+	'string':   StringType,
+	'int':      IntType,
+	'bool':     IntType,
+	'long':     LongType,
+	'decimal':  FloatType,
+	'float':    FloatType,
+	'datetime': DateTimeType,
+	'date':     DateTimeType,
+	'object':   ObjectType
 }
 
 
@@ -265,20 +252,20 @@ class DataTableError(Exception):
 
 
 class TableColumn:
-	"""
-	A table column represents a column of the table including name and
-	type.
+	"""Representation of a table column.
+
+	A table column represents a column of the table including name and type.
+	It does not contain the actual values of the column. These are stored
+	individually in the rows.
 
-	It does not contain the actual values of the column. These are
-	stored individually in the rows.
 	"""
 
 	def __init__(self, spec):
 
 		# spec is a string such as 'name' or 'name:type'
-		fields = split(spec, ':')
+		fields = spec.split(':', 2)
 		if len(fields) > 2:
-			raise DataTableError, 'Invalid column spec %s' % repr(spec)
+			raise DataTableError, 'Invalid column spec %r' % spec
 		self._name = fields[0]
 
 		if len(fields) == 1:
@@ -293,18 +280,23 @@ class TableColumn:
 		return self._type
 
 	def setType(self, type):
-		""" Sets the type (by a string containing the name) of the heading. Usually invoked by DataTable to set the default type for columns whose types were not specified. """
-		if type == None:
+		"""Set the type (by a string containing the name) of the heading.
+
+		Usually invoked by DataTable to set the default type for columns
+		whose types were not specified.
+
+		"""
+		if type is None:
 			self._type = None
 		else:
 			try:
 				self._type = _types[type.lower()]
 			except:
-				raise DataTableError, 'Unknown type %s. types=%r' % (repr(type), _types.keys())
+				raise DataTableError, 'Unknown type %r. types=%r' % (type, _types.keys())
 
 	def __repr__(self):
-		return '<%s %s with %s at %x>' % (
-			self.__class__.__name__, repr(self._name), repr(self._type), id(self))
+		return '<%s %r with %r at %x>' % (
+			self.__class__.__name__, self._name, self._type, positive_id(self))
 
 	def __str__(self):
 		return self._name
@@ -313,8 +305,16 @@ class TableColumn:
 	## Utilities ##
 
 	def valueForRawValue(self, rawValue):
-		""" The rawValue is typically a string or value already of the appropriate type. TableRecord invokes this method to ensure that values (especially strings that come from files) are the correct types (e.g., ints are ints and floats are floats). """
-		# @@ 2000-07-23 ce: an if-else ladder? perhaps these should be dispatched messages or a class hier
+		"""Set correct type for raw value.
+
+		The rawValue is typically a string or value already of the appropriate
+		type. TableRecord invokes this method to ensure that values (especially
+		strings that come from files) are the correct types (e.g., ints are
+		ints and floats are floats).
+
+		"""
+		# @@ 2000-07-23 ce: an if-else ladder?
+		# perhaps these should be dispatched messages or a class hier
 		if self._type is StringType:
 			value = str(rawValue)
 		elif self._type is IntType:
@@ -333,7 +333,7 @@ class TableColumn:
 			else:
 				value = float(rawValue)
 		elif self._type is DateTimeType:
-			value = DateTimeFrom(rawValue)
+			value = mxDateTime.DateTimeFrom(rawValue)
 		elif self._type is ObjectType:
 			value = rawValue
 		else:
@@ -343,22 +343,26 @@ class TableColumn:
 
 
 class DataTable:
-	"""
+	"""Representation of a data table.
+
 	See the doc string for this module.
+
 	"""
 
-	usePickleCache = True
+	_usePickleCache = True
 
 
 	## Init ##
 
-	def __init__(self, filenameOrHeadings=None, delimiter=',', allowComments=1, stripWhite=1, defaultType=None, usePickleCache=None):
+	def __init__(self, filenameOrHeadings=None, delimiter=',',
+			allowComments=True, stripWhite=True,
+			defaultType=None, usePickleCache=None):
 		if usePickleCache is None:
-			self.usePickleCache = self.usePickleCache  # grab the class-level attr
+			self._usePickleCache = self._usePickleCache # grab class-level attr
 		else:
-			self.usePickleCache = usePickleCache
+			self._usePickleCache = usePickleCache
 		if defaultType and not _types.has_key(defaultType):
-			raise DataTableError, 'Unknown type for default type: %s' % repr(defaultType)
+			raise DataTableError, 'Unknown type for default type: %r' % defaultType
 		self._defaultType = defaultType
 		self._filename = None
 		self._headings = []
@@ -372,10 +376,11 @@ class DataTable:
 
 	## File I/O ##
 
-	def readFileNamed(self, filename, delimiter=',', allowComments=1, stripWhite=1, worksheet=1, row=1, column=1):
+	def readFileNamed(self, filename, delimiter=',',
+			allowComments=True, stripWhite=True, worksheet=1, row=1, column=1):
 		self._filename = filename
 		data = None
-		if self.usePickleCache:
+		if self._usePickleCache:
 			from PickleCache import readPickleCache, writePickleCache
 			data = readPickleCache(filename, pickleVersion=1, source='MiscUtils.DataTable')
 		if data is None:
@@ -385,23 +390,29 @@ class DataTable:
 				file = open(self._filename, 'r')
 				self.readFile(file, delimiter, allowComments, stripWhite)
 				file.close()
-			if self.usePickleCache:
+			if self._usePickleCache:
 				writePickleCache(self, filename, pickleVersion=1, source='MiscUtils.DataTable')
 		else:
 			self.__dict__ = data.__dict__
 		return self
 
-	def readFile(self, file, delimiter=',', allowComments=1, stripWhite=1):
-		return self.readLines(file.readlines(), delimiter, allowComments, stripWhite)
+	def readFile(self, file, delimiter=',',
+			allowComments=True, stripWhite=True):
+		return self.readLines(file.readlines(), delimiter,
+			allowComments, stripWhite)
 
-	def readString(self, string, delimiter=',', allowComments=1, stripWhite=1):
-		return self.readLines(split(string, '\n'), delimiter, allowComments, stripWhite)
+	def readString(self, string, delimiter=',',
+			allowComments=True, stripWhite=True):
+		return self.readLines(string.split('\n'), delimiter,
+			allowComments, stripWhite)
 
-	def readLines(self, lines, delimiter=',', allowComments=1, stripWhite=1):
+	def readLines(self, lines, delimiter=',',
+			allowComments=True, stripWhite=True):
 		if self._defaultType is None:
 			self._defaultType = 'string'
 		haveReadHeadings = False
-		parse = CSVParser(fieldSep=delimiter, allowComments=allowComments, stripWhitespace=stripWhite).parse
+		parse = CSVParser(fieldSep=delimiter, allowComments=allowComments,
+			stripWhitespace=stripWhite).parse
 		for line in lines:
 			# process a row, either headings or data
 			values = parse(line)
@@ -449,24 +460,24 @@ class DataTable:
 			haveReadHeadings = False
 			rowNum = row
 			numBlankRows = 0
-			valuesBuffer = {}   # keyed by row number
+			valuesBuffer = {} # keyed by row number
 			while 1:
 				try:
 					# grab a single row
 					values = valuesBuffer[rowNum]
 				except KeyError:
 					# woops. read buffer is out of fresh rows
-					valuesRows = sh.Range('A%i:%s%i' % (rowNum, maxCol, rowNum+numRowsToReadPerCall-1)).Value
+					valuesRows = sh.Range('A%i:%s%i' % (rowNum, maxCol,
+						rowNum+numRowsToReadPerCall-1)).Value
 					valuesBuffer.clear()
 					j = rowNum
 					for valuesRow in valuesRows:
 						valuesBuffer[j] = valuesRow
 						j += 1
 					values = valuesBuffer[rowNum]
-
 				# non-"buffered" version, one row at a time:
-				# values = sh.Range('A%i:%s%i' % (rowNum, maxCol, rowNum)).Value[0]
-
+				# values = sh.Range('A%i:%s%i' % (rowNum, maxCol,
+				# rowNum)).Value[0]
 				values = [strip(v) for v in values]
 				nonEmpty = [v for v in values if v]
 				if nonEmpty:
@@ -496,16 +507,18 @@ class DataTable:
 		file.close()
 
 	def writeFile(self, file):
-		"""
-		@@ 2000-07-20 ce: This doesn't write the column types (like :int) back out.
-		@@ 2000-07-21 ce: It's notable that a blank numeric value gets read as zero and written out that way. Also, values None are written as blanks.
-		"""
+		"""Write the table out as a file.
 
+		@@ 2000-07-20 ce: This doesn't write the column types (like :int) back out.
+		@@ 2000-07-21 ce: It's notable that a blank numeric value gets read as zero
+			and written out that way. Also, values None are written as blanks.
+
+		"""
 		# write headings
-		file.write(join(map(lambda h: str(h), self._headings), ','))
+		file.write(','.join(map(str, self._headings)))
 		file.write('\n')
 
-		def ValueWritingMapper(item):
+		def valueWritingMapper(item):
 			# So that None gets written as a blank and everything else as a string
 			if item is None:
 				return ''
@@ -514,7 +527,7 @@ class DataTable:
 
 		# write rows
 		for row in self._rows:
-			file.write(joinCSVFields(map(ValueWritingMapper, row)))
+			file.write(joinCSVFields(map(valueWritingMapper, row)))
 			file.write('\n')
 
 	def commit(self):
@@ -540,10 +553,11 @@ class DataTable:
 		return self._headings
 
 	def setHeadings(self, headings):
-		""" Headings can be a list of strings (like ['name', 'age:int']) or a list of TableColumns or None. """
+		"""Headings can be a list of strings (like ['name', 'age:int'])
+		or a list of TableColumns or None. """
 		if not headings:
 			self._headings = []
-		elif isinstance(headings[0], StringTypes):
+		elif type(headings[0]) in StringTypes:
 			self._headings = map(lambda h: TableColumn(h), headings)
 		elif isinstance(headings[0], TableColumn):
 			self._headings = list(headings)
@@ -562,8 +576,12 @@ class DataTable:
 		return self._rows[index]
 
 	def append(self, object):
-		""" If object is not a TableRecord, then one is created, passing the object to initialize the TableRecord. Therefore, object can be a TableRecord, list, dictionary or object. See TableRecord for details. """
+		"""If object is not a TableRecord, then one is created,
+		passing the object to initialize the TableRecord.
+		Therefore, object can be a TableRecord, list, dictionary or object.
+		See TableRecord for details.
 
+		"""
 		if not isinstance(object, TableRecord):
 			object = TableRecord(self, object)
 		self._rows.append(object)
@@ -578,7 +596,7 @@ class DataTable:
 		for record in self._rows:
 			matches = True
 			for key in keys:
-				if record[key]!=dict[key]:
+				if record[key] != dict[key]:
 					matches = False
 					break
 			if matches:
@@ -591,26 +609,30 @@ class DataTable:
 	def __repr__(self):
 		# Initial info
 		s = ['DataTable: %s\n%d rows\n' % (self._filename, len(self._rows))]
-
 		# Headings
 		s.append('     ')
-		s.append(join(map(lambda h: str(h), self._headings), ', '))
+		s.append(', '.join(map(str, self._headings)))
 		s.append('\n')
-
 		# Records
 		i = 0
 		for row in self._rows:
 			s.append('%3d. ' % i)
-			s.append(join(map(lambda r: str(r), row), ', '))
+			s.append(', '.join(map(str, row)))
 			s.append('\n')
-			i = i + 1
-		return join(s, '')
+			i += 1
+		return ''.join(s)
 
 
 	## As a dictionary ##
 
 	def dictKeyedBy(self, key):
-		""" Returns a dictionary containing the contents of the table indexed by the particular key. This is useful for tables that have a column which represents a unique key (such as a name, serial number, etc.). """
+		"""Return a dictionary containing the contents of the table.
+
+		The content is indexed by the particular key. This is useful
+		for tables that have a column which represents a unique key
+		(such as a name, serial number, etc.).
+
+		"""
 		dict = {}
 		for row in self:
 			dict[row[key]] = row
@@ -623,16 +645,23 @@ class DataTable:
 		return self._filename
 
 	def nameToIndexMap(self):
-		""" Table rows keep a reference to this map in order to speed up index-by-names (as in row['name']). """
+		"""Speed-up index.
+
+		Table rows keep a reference to this map in order to speed up
+		index-by-names (as in row['name']).
+
+		"""
 		return self._nameToIndexMap
 
 
 	## Self utilities ##
 
 	def createNameToIndexMap(self):
-		"""
+		"""Create speed-up index.
+
 		Invoked by self to create the nameToIndexMap after the table's
 		headings have been read/initialized.
+
 		"""
 		map = {}
 		for i in range(len(self._headings)):
@@ -640,7 +669,8 @@ class DataTable:
 		self._nameToIndexMap = map
 
 
-# @@ 2000-07-20 ce: perhaps for each type we could specify a function to convert from string values to the values of the type
+# @@ 2000-07-20 ce: perhaps for each type we could specify a function
+# to convert from string values to the values of the type.
 
 BlankValues = {
 	StringType:   '',
@@ -652,20 +682,26 @@ BlankValues = {
 
 
 class TableRecord:
+	"""Representation of a table record."""
+
 
 	## Init ##
 
 	def __init__(self, table, values=None):
-		"""
-		Dispatches control to one of the other init methods based on the type of values.  Values can be one of three things:
+		"""Initialize table record.
+
+		Dispatches control to one of the other init methods based on the type
+		of values. Values can be one of three things:
 			1. A TableRecord
 			2. A list
 			3. A dictionary
 			4. Any object responding to hasValueForKey() and valueForKey().
+
 		"""
 		self._headings = table.headings()
 		self._nameToIndexMap = table.nameToIndexMap()
-		# @@ 2000-07-20 ce: Take out the headings arg to the init method since we have an attribute for that
+		# @@ 2000-07-20 ce: Take out the headings arg to the init method
+		# since we have an attribute for that
 
 		if values is not None:
 			valuesType = type(values)
@@ -681,7 +717,9 @@ class TableRecord:
 
 	def initFromSequence(self, values):
 		if len(self._headings) < len(values):
-			raise DataTableError, ('There are more values than headings.\nheadings(%d, %s)\nvalues(%d, %s)' % (len(self._headings), self._headings, len(values), values))
+			raise DataTableError, ('There are more values than headings.\n'
+				'headings(%d, %s)\nvalues(%d, %s)' % (len(self._headings),
+				self._headings, len(values), values))
 		self._values = []
 		numHeadings = len(self._headings)
 		numValues = len(values)
@@ -703,19 +741,22 @@ class TableRecord:
 				self._values.append(BlankValues[heading.type()])
 
 	def initFromObject(self, object):
-		"""
+		"""Initialize from object.
+
 		The object is expected to response to hasValueForKey(name) and
 		valueForKey(name) for each of the headings in the table. It's
 		alright if the object returns False for hasValueForKey(). In that
 		case, a "blank" value is assumed (such as zero or an empty
 		string). If hasValueForKey() returns True, then valueForKey() must
 		return a value.
+
 		"""
 		self._values = []
 		for heading in self._headings:
 			name = heading.name()
 			if object.hasValueForKey(name):
-				self._values.append(heading.valueForRawValue(object.valueForKey(name)))
+				self._values.append(heading.valueForRawValue(
+					object.valueForKey(name)))
 			else:
 				self._values.append(BlankValues[heading.type()])
 
@@ -726,12 +767,13 @@ class TableRecord:
 		return len(self._values)
 
 	def __getitem__(self, key):
-		if isinstance(key, StringTypes):
+		if type(key) in StringTypes:
 			key = self._nameToIndexMap[key]
 		try:
 			return self._values[key]
 		except TypeError:
-			raise TypeError, 'key=%r, key type=%r, self._values=%r' % (key, type(key), self._values)
+			raise TypeError, 'key=%r, key type=%r, self._values=%r' % (
+				key, type(key), self._values)
 
 	def __setitem__(self, key, value):
 		if type(key) is StringType:
@@ -767,15 +809,16 @@ class TableRecord:
 	## Additional access ##
 
 	def asList(self):
-		"""
-		Returns a sequence whose values are the same at the record's
-		and in the order defined by the table.
+		"""Return a sequence whose values are the same as the record's.
+
+		The order of the sequence is the one defined by the table.
+
 		"""
 		# It just so happens that our implementation already has this
 		return self._values[:]
 
 	def asDict(self):
-		""" Returns a dictionary whose key-values match the table record. """
+		"""Return a dictionary whose key-values match the table record."""
 		dict = {}
 		nameToIndexMap = self._nameToIndexMap
 		for key in nameToIndexMap.keys():
