@@ -1,6 +1,7 @@
-from WebKit.Examples.ExamplePage import ExamplePage
 import os
 from glob import glob
+from WebKit.Examples.ExamplePage import ExamplePage
+
 
 class PSPExamplePage(ExamplePage):
 
@@ -21,12 +22,12 @@ class PSPExamplePage(ExamplePage):
 
 	def writeOtherMenu(self):
 		self.menuHeading('Other')
-		viewPath = self.request().uriWebKitRoot() + "PSPExamples/View"
+		viewPath = self.request().uriWebKitRoot() + "PSP/Examples/View"
 		self.menuItem(
 			'View source of<br/>%s' % self.title(),
-			self.request().uriWebKitRoot() + 'PSPExamples/View?filename=%s' \
+			self.request().uriWebKitRoot() + 'PSP/Examples/View?filename=%s' \
 				% os.path.basename(self.request().serverSidePath()))
 		if self.application().hasContext('Documentation'):
 			filename = 'Documentation/WebKit.html'
 			if os.path.exists(filename):
-				self.menuItem('Local WebKit docs', self.request().adapterName() + '/' + filename)
+				self.menuItem('Local WebKit docs', self.request().servletPath() + '/' + filename)
