@@ -1,6 +1,7 @@
-from AdminSecurity import *
+from AdminSecurity import AdminSecurity
 from WebUtils import Funcs
 
+
 class Config(AdminSecurity):
 
 	def title(self):
