@@ -97,7 +97,6 @@ class AppServer(ConfigurableForServerSidePath, Object):
 		self.printStartUpMessage()
 		sys.setcheckinterval(self.setting('CheckInterval'))
 		self._app = self.createApplication()
-		self.makeDirs()
 		self.loadPlugIns()
 
 		# @@ 2003-03 ib: shouldn't this just be in a subclass's __init__?
@@ -126,13 +125,6 @@ class AppServer(ConfigurableForServerSidePath, Object):
 			print
 			sys.exit(0)
 
-	def makeDirs(self):
-		"""Make sure some standard directories are always available."""
-		for dir in ('Cache', 'Sessions', 'ErrorMsgs', 'Logs'):
-			dir = os.path.join(self.serverSidePath(), dir)
-			if not os.path.exists(dir):
-				os.mkdir(dir)
-
 	def readyForRequests(self):
 		"""Declare ready for getting requests.
 
@@ -325,7 +317,7 @@ class AppServer(ConfigurableForServerSidePath, Object):
 					and filename not in plugIns):
 					plugIns.append(filename)
 
-		print 'Plug-ins list:', ', '.join(plugIns)
+		print 'Plug-ins list:', ', '.join(plugIns) or 'empty'
 
 		# Now that we have our plug-in list, load them...
 		for plugInPath in plugIns:
