@@ -48,11 +48,15 @@ defaultConfig = {
 		],
 	'SessionModule': 'Session',
 	'SessionStore': 'Dynamic',
+	'SessionStoreDir': 'Sessions',
 	'SessionTimeout': 60,
 	'SessionPrefix': '',
 	'SessionName': '_SID_',
 	'IgnoreInvalidSession': True,
 	'UseAutomaticPathSessions': False,
+	'UseCookieSessions': True,
+	'SessionCookiePath': None,
+	'SecureSessionCookie': True,
 	'ShowDebugInfoOnErrors': True,
 	'IncludeFancyTraceback': False,
 	'FancyTracebackContext': 5,
@@ -77,6 +81,7 @@ defaultConfig = {
 	'MaxValueLengthInExceptionReport': 500,
 	'RPCExceptionReturn': 'traceback',
 	'ReportRPCExceptionsInWebKit': True,
+	'CacheDir': 'Cache',
 	'Contexts': {
 		'default': 'Examples',
 		'Admin': 'Admin',
@@ -171,6 +176,7 @@ class Application(ConfigurableForServerSidePath, Object):
 		self._exceptionHandlerClass = ExceptionHandler
 
 		self.initSessions()
+		self.makeDirs()
 
 		URLParser.initApp(self)
 		self._rootURLParser = URLParser.ContextParser(self)
@@ -241,6 +247,9 @@ class Application(ConfigurableForServerSidePath, Object):
 		moduleName = self.setting('SessionStore')
 		if moduleName in ('Dynamic', 'File', 'Memory'):
 			moduleName = 'Session%sStore' % moduleName
+		self._sessionDir = moduleName != 'SessionMemoryStore' \
+			and self.serverSidePath(
+				self.setting('SessionStoreDir') or 'Sessions') or None
 		className = moduleName.split('.')[-1]
 		try:
 			exec 'from %s import %s' % (moduleName, className)
@@ -254,6 +263,17 @@ class Application(ConfigurableForServerSidePath, Object):
 				" from module '%s'" % (className, moduleName)
 			self._sessions = None
 
+	def makeDirs(self):
+		"""Make sure some standard directories are always available."""
+		self._cacheDir = self.serverSidePath(
+			self.setting('CacheDir') or 'Cache')
+		self._errorMessagesDir = self.serverSidePath(
+			self.setting('ErrorMessagesDir') or 'ErrorMsgs')
+		for dir in (self.serverSidePath('Logs'),
+				self._cacheDir, self._errorMessagesDir, self._sessionDir):
+			if dir and not os.path.exists(dir):
+				os.makedirs(dir)
+
 	def initVersions(self):
 		"""Get and store versions.
 
@@ -454,11 +474,13 @@ class Application(ConfigurableForServerSidePath, Object):
 	def sessionCookiePath(self, trans):
 		"""Get the cookie path for this transaction.
 
-		The servlet path is used for security reasons, see:
+		If not path is specified in the configuration setting,
+		the servlet path is used for security reasons, see:
 		http://www.net-security.org/dl/articles/cookie_path.pdf
 
 		"""
-		return trans.request().servletPath() + '/'
+		return self.setting('SessionCookiePath') or (
+			trans.request().servletPath() + '/')
 
 
 	## Misc Access ##
