@@ -3,7 +3,7 @@
 """AutoReloadingAppServer
 
 This module defines `AutoReloadingAppServer`, a replacement for `AppServer`
-that adds a file-monitoring and restarting to the AppServer. Used mostly like::
+that adds a file-monitoring and restarting to the AppServer. Used mostly like:
 
     from AutoReloadingAppServer import AutoReloadingAppServer as AppServer
 
@@ -13,28 +13,133 @@ in seconds can be set with `AutoReloadPollInterval` in AppServer.config.
 
 """
 
-import select
+import select, errno
 from threading import Thread
 
 from Common import *
 from AppServer import AppServer
 
-# Attempt to use python-fam (FAM = File Alteration Monitor) instead of polling
-# to see if files have changed. Get it from http://python-fam.sourceforge.net.
-# If FAM is not available or ImportSpy is not used, we fall back to polling.
-try:
-	import _fam
-except ImportError:
-	_fam = None
-
 
 defaultConfig = {
 	'AutoReload': False,
 	'AutoReloadPollInterval': 1, # in seconds
 	'UseImportSpy': True,
+	'UseFAMModules': 'gamin _fam',
 }
 
 
+def fam(modules):
+	"""Get FAM object based on the modules specified.
+
+	Currently supported are
+	python-gamin (gamin): http://www.gnome.org/~veillard/gamin/
+	python-fam (_fam): http://python-fam.sourceforge.net
+
+	"""
+	for module in modules:
+		try:
+			fam = __import__(module)
+		except ImportError:
+			fam = None
+		if not fam:
+			continue
+		if hasattr(fam, 'GAM_CONNECT') and hasattr(
+				fam, 'WatchMonitor') and hasattr(fam, 'GAMChanged'):
+
+
+			class FAM:
+				"""Simple File Alteration Monitor based on python-gamin"""
+
+				def name(self):
+					return "python-gamin"
+
+				def __init__(self):
+					"""Initialize and start monitoring."""
+					self._mon = fam.WatchMonitor()
+					self._watchlist = []
+
+				def close(self):
+					"""Stop monitoring and close."""
+					for filepath in self._watchlist:
+						self._mon.stop_watch(filepath)
+					self._mon.disconnect()
+
+				def fd(self):
+					"""Get file descriptor for monitor."""
+					return self._mon.get_fd()
+
+				def monitorFile(self, filepath):
+					"""Monitor one file."""
+					self._mon.watch_file(filepath, self.callback)
+					self._watchlist.append(filepath)
+
+				def pending(self):
+					"""Check whether an event is pending."""
+					return self._mon.event_pending()
+
+				def nextFile(self):
+					"""Get next file and return whether it has changed."""
+					self._mon.handle_one_event()
+					return self._changed, self._filepath
+
+				def callback(self, filepath, event):
+					"""Callback function for WatchMonitor."""
+					self._filepath = filepath
+					self._changed = {fam.GAMChanged: 'changed',
+						fam.GAMCreated: 'created', fam.GAMMoved: 'moved',
+						fam.GAMDeleted: 'deleted'}.get(event)
+
+			break
+		elif hasattr(fam, 'FAMConnection') and hasattr(
+				fam, 'open') and hasattr(fam, 'Changed'):
+
+
+			class FAM:
+				"""Simple File Alteration Monitor based on python-fam"""
+
+				def name(self):
+					return "python-fam"
+
+				def __init__(self):
+					"""Initialize and start monitoring."""
+					self._fc = fam.open()
+					self._requests = []
+
+				def close(self):
+					"""Stop monitoring and close."""
+					for request in self._requests:
+						request.cancelMonitor()
+					self._fc.close()
+
+				def fd(self):
+					"""Get file descriptor for monitor."""
+					return self._fc
+
+				def monitorFile(self, filepath):
+					"""Monitor one file."""
+					self._requests.append(self._fc.monitorFile(filepath, None))
+
+				def pending(self):
+					"""Check whether an event is pending."""
+					return self._fc.pending()
+
+				def nextFile(self):
+					"""Get next file and return whether it has changed."""
+					event = self._fc.nextEvent()
+					# we can also use event.code2str() here,
+					# but then we need to filter the events
+					changed = {fam.Changed: 'changed',
+						fam.Created: 'created', fam.Moved: 'moved',
+						fam.Deleted: 'deleted'}.get(event.code)
+					return changed, event.filename
+
+			break
+	else:
+		FAM = None
+	if FAM:
+		return FAM()
+
+
 class AutoReloadingAppServer(AppServer):
 	"""AppServer AutoReloading.
 
@@ -55,13 +160,15 @@ class AutoReloadingAppServer(AppServer):
 
 	def __init__(self, path=None):
 		"""Activate AutoReloading."""
-		AppServer.__init__(self, path)
-		self._autoReload = False
 		self._shouldRestart = False
-		self._useFAM = False
-		if self.isPersistent():
-			if self.setting('AutoReload'):
+		self._fileMonitorThread = None
+		AppServer.__init__(self, path)
+		try:
+			if self.isPersistent() and self.setting('AutoReload'):
 				self.activateAutoReload()
+		except:
+			AppServer.initiateShutdown(self)
+			raise
 
 	def defaultConfig(self):
 		"""Return the default configuration."""
@@ -78,13 +185,8 @@ class AutoReloadingAppServer(AppServer):
 
 		"""
 		print 'Stopping AutoReload Monitor...'
-		sys.stdout.flush()
-		self._running = 1
 		self.deactivateAutoReload()
 		AppServer.shutDown(self)
-		sys.stdout.flush()
-		sys.stderr.flush()
-		self._running = 0
 
 
 	## Activation of AutoReload ##
@@ -94,47 +196,60 @@ class AutoReloadingAppServer(AppServer):
 		if self.setting('UseImportSpy'):
 			s = self._imp.activateImportSpy()
 			print 'ImportSpy activated (using %s).' % s
-		self._useFAM = False
-		if not self._autoReload:
-			if _fam and self._imp._spy:
+		if not self._fileMonitorThread:
+			famModules = self.setting('UseFAMModules')
+			try:
+				famModules = famModules.split()
+			except AttributeError:
+				pass
+			if famModules and self._imp._spy:
 				# FAM will be only used when ImportSpy has been activated,
 				# since otherwise we need to poll the modules anyway.
 				try:
-					self._fc = _fam.open()
+					self._fam = fam(famModules)
 					self._pipe = None
-					self._useFAM = True
-				except IOError:
+				except Exception, e:
+					print "Error loading FAM:", str(e)
+					self._fam = None
+				if not self._fam:
 					print 'FAM not available, fall back to polling.'
-					self._fc = None
-			if self._useFAM:
-				print 'AutoReload Monitor started, using FAM.'
+			else:
+				self._fam = None
+			print 'AutoReload Monitor started,',
+			if self._fam:
+				print 'using %s.' % self._fam.name()
 				target = self.fileMonitorThreadLoopFAM
 			else:
 				self._pollInterval = self.setting('AutoReloadPollInterval')
-				print 'AutoReload Monitor started, ' \
-					'polling every %d seconds.' % self._pollInterval
+				print 'polling every %d seconds.' % self._pollInterval
 				target = self.fileMonitorThreadLoop
+			self._runFileMonitor = True
 			self._fileMonitorThread = t = Thread(target=target)
-			self._autoReload = True
 			t.setName('AutoReloadMonitor')
 			t.start()
 
 	def deactivateAutoReload(self):
-		"""Tell the monitor thread to stop.
-
-		This should be considered as a request, not a demand.
-
-		"""
-		self._autoReload = False
-		if self._useFAM and self._pipe:
-			# Send a message down the pipe to wake up the monitor thread
-			# and tell him to quit.
-			self._pipe[1].write('close')
-			self._pipe[1].flush()
-		try:
-			self._fileMonitorThread.join()
-		except Exception:
-			pass
+		"""Stop the monitor thread."""
+		if self._fileMonitorThread:
+			if self._runFileMonitor:
+				self._runFileMonitor = False
+				if self._fam:
+					if self._pipe:
+						# Send a message down the pipe to wake up the monitor thread
+						# and tell him to quit.
+						os.write(self._pipe[1], 'stop')
+						os.close(self._pipe[1])
+			sys.stdout.flush()
+			try:
+				self._fileMonitorThread.join()
+			except Exception:
+				pass
+			if self._fam:
+				if self._pipe:
+					os.close(self._pipe[0])
+					self._pipe = None
+				self._fam.close()
+				self._fam = None
 
 
 	## Restart methods ##
@@ -148,16 +263,6 @@ class AutoReloadingAppServer(AppServer):
 		has been called.
 
 		"""
-		# Tavis Rudd claims: "this method can only be called by
-		# the main thread.  If a worker thread calls it, the
-		# process will freeze up."
-		#
-		# I've implemented it so that the ThreadedAppServer's
-		# control thread calls this. That thread is _not_ the
-		# MainThread (the initial thread created by the Python
-		# interpreter), but I've never encountered any problems.
-		# Most likely Tavis meant a freeze would occur if a
-		# _worker_ called this.
 		if self._shouldRestart:
 			self.restart()
 
@@ -184,16 +289,15 @@ class AutoReloadingAppServer(AppServer):
 		to monitor. This is only used when we are using FAM.
 
 		"""
-		filepath = os.path.abspath(filepath)
-		self._requests.append(self._fc.monitorFile(filepath, filepath))
+		self._fam.monitorFile(os.path.abspath(filepath))
 
 
 	## Internal methods ##
 
 	def shouldRestart(self):
 		"""Tell the main thread to restart the server."""
-		self._autoReload = False
 		self._shouldRestart = True
+		self._runFileMonitor = False
 
 	def fileMonitorThreadLoop(self):
 		"""This the the main loop for the monitoring thread.
@@ -204,65 +308,47 @@ class AutoReloadingAppServer(AppServer):
 		was initially loaded).
 
 		"""
-		while self._autoReload:
+		while self._runFileMonitor:
 			time.sleep(self._pollInterval)
 			f = self._imp.updatedFile()
 			if f:
 				print '*** The file', f, 'has changed.'
-				print 'The app server is restarting now...'
+				print 'Restarting AppServer...'
 				self.shouldRestart()
-				return
-		print 'Autoreload Monitor stopped.'
-		sys.stdout.flush()
 
 	def fileMonitorThreadLoopFAM(self, getmtime=os.path.getmtime):
 		"""Monitoring thread loop, but using the FAM library."""
-		self._pipe = self._requests = None
 		# For all of the modules which have _already_ been loaded,
 		# we check to see if they've already been modified:
 		f = self._imp.updatedFile()
 		if f:
 			print '*** The file', f, 'has changed.'
-			print 'The app server is restarting now...'
+			print 'Restarting AppServer...'
 			self.shouldRestart()
 			return
-		self._requests = []
-		for f in self._imp.fileList():
+		for f in self._imp.fileList().keys():
 			self.monitorNewModule(f)
 		self._imp.notifyOfNewFiles(self.monitorNewModule)
 		# Create a pipe so that this thread can be notified when the
 		# server is shutdown. We use a pipe because it needs to be an object
 		# which will wake up the call to 'select':
-		r, w = os.pipe()
-		r, w = os.fdopen(r, 'r'), os.fdopen(w, 'w')
-		self._pipe = pipe = (r, w)
-		fc = self._fc
-		while self._autoReload:
+		self._pipe = os.pipe()
+		fds = [self._fam.fd(), self._pipe[0]], [], []
+		while self._runFileMonitor:
 			try:
 				# We block here until a file has been changed, or until
 				# we receive word that we should shutdown (via the pipe).
-				ri, ro, re = select.select([fc, pipe[0]], [], [])
-			except select.error, er:
-				errnumber, strerr = er
-				if errnumber == errno.EINTR:
+				select.select(*fds)
+			except select.error, e:
+				if e[0] == errno.EINTR:
 					continue
 				else:
-					print strerr
+					print "Error:", e[1]
 					sys.exit(1)
-			while fc.pending():
-				fe = fc.nextEvent()
-				c, f = fe.code2str(), fe.userData
-				if c in ('changed', 'deleted', 'created') \
-						and self._imp.fileUpdated(f):
+			while self._runFileMonitor and self._fam.pending():
+				c, f = self._fam.nextFile()
+				if c and self._imp.fileUpdated(f):
 					print '*** The file %s has been %s.' % (f, c)
-					print 'The app server is restarting now...'
+					print 'Restarting AppServer...'
 					self.shouldRestart()
-					return
 		self._imp.notifyOfNewFiles(None)
-		self._pipe = None
-		for req in self._requests:
-			req.cancelMonitor()
-		self._requests = None
-		fc.close()
-		print 'Autoreload Monitor stopped'
-		sys.stdout.flush()
