@@ -73,8 +73,7 @@ class HTTPRequest(Request):
 			# correct servletPath if there was a redirection
 			if not (self._uri + '/').startswith(self._servletPath + '/'):
 				i = self._uri.find(self._pathInfo)
-				if i >= 0:
-					self._servletPath = self._uri[:i]
+				self._servletPath = i >= 0 and self._uri[:i] or '/'
 		else:
 			# REQUEST_URI isn't actually part of the CGI standard and some
 			# web servers like IIS don't set it (as of 8/22/2000).
@@ -83,8 +82,7 @@ class HTTPRequest(Request):
 				# correct servletPath if there was a redirection
 				if not (self._uri + '/').startswith(self._servletPath + '/'):
 					i = self._uri.find(self._pathInfo)
-					if i >= 0:
-						self._servletPath = self._uri[:i]
+					self._servletPath = i >= 0 and self._uri[:i] or '/'
 			else:
 				self._uri = self._servletPath + self._pathInfo
 			if self._queryString:
@@ -138,6 +136,13 @@ class HTTPRequest(Request):
 			print "Done setting up request, found keys %r" % self._fields.keys()
 
 
+	## Security ##
+
+	def isSecure(self):
+		"""Check whether this is a HTTPS connection."""
+		return self._environ.get('HTTPS', '').lower() == 'on'
+
+
 	## Transactions ##
 
 	def responseClass(self):
@@ -402,22 +407,21 @@ class HTTPRequest(Request):
 			fspath = os.path.join(docroot, fspath)
 		return fspath
 
-	def serverURL(self):
+	def serverURL(self, canonical=False):
 		"""Return the full internet path to this request.
 
 		This is the URL that was actually received by the webserver
-		before any rewriting took place.
+		before any rewriting took place. If canonical is set to true,
+		then the canonical hostname of the server is used if possible.
 
 		The path is returned without any extra path info or query strings,
 		i.e. http://www.my.own.host.com:8080/WebKit/TestPage.py
 
 		"""
-		if self._environ.has_key('SCRIPT_URI'):
+		if canonical and self._environ.has_key('SCRIPT_URI'):
 			return self._environ['SCRIPT_URI']
 		else:
-			scheme = 'http'
-			if self._environ.get('HTTPS', '').lower() == 'on':
-				scheme += 's'
+			scheme = self.isSecure() and 'https' or 'http'
 			host = self._environ['HTTP_HOST'] # includes port
 			return scheme + '://' + host + self.serverPath()
 
@@ -615,17 +619,33 @@ class HTTPRequest(Request):
 	def originalURI(self):
 		"""Get URI of the original servlet before any forwarding."""
 		if self._stack:
-			return self._stack[0][2]
+			return self._servletPath + self._stack[0][1]
 		else:
 			return self.uri()
 
 	def previousURI(self):
 		"""Get the previous URI, if any."""
 		if self._stack:
-			return self._stack[-1][2]
+			return self._servletPath + self._stack[-1][1]
 
 	def previousURIs(self):
 		"""Get the list of all previous URIs."""
+		return [self._servletPath + s[1] for s in self._stack]
+
+	def originalContextName(self):
+		"""Return the name of the original context before any forwarding."""
+		if self._stack:
+			return self._stack[0][2]
+		else:
+			return self._contextName
+
+	def previousContextName(self):
+		"""Get the previous context name, if any."""
+		if self._stack:
+			return self._stack[-1][2]
+
+	def previousContextNames(self):
+		"""Get the list of all previous context names."""
 		return [s[2] for s in self._stack]
 
 	def rawInput(self, rewind=False):
