@@ -156,7 +156,7 @@ class MakeAppWorkDir:
 	def copyOtherFiles(self):
 		"""Make a copy of any other necessary files in the new work dir."""
 		self.msg("Copying other files...")
-		otherFiles = ('AppServer', 'webkit', '404Text.txt')
+		otherFiles = ('AppServer', 'webkit', 'error404.html')
 		for name in otherFiles:
 			if name == 'AppServer':
 				if self._osType == 'nt':
@@ -203,7 +203,9 @@ class MakeAppWorkDir:
 			import grp
 			group = grp.getgrgid(gid)[0]
 		executable = sys.executable
-		for name in launcherScripts:
+		scriptNames = launcherScripts.keys()
+		scriptNames.sort()
+		for name in scriptNames:
 			if name.endswith('Service.py') and self._osType != 'nt':
 				continue
 			newname = os.path.join(workDir, name)
@@ -252,9 +254,8 @@ class MakeAppWorkDir:
 			self._contextDir or self._contextName)
 		if contextDir.startswith(self._workDir):
 			configDir = contextDir[len(self._workDir):]
-			configDir = configDir.lstrip(os.sep)
-			if os.altsep:
-				configDir =  configDir.lstrip(os.altsep)
+			while configDir[:1] in (os.sep, os.altsep):
+				configDir = configDir[1:]
 		else:
 			configDir = contextDir
 		if os.path.exists(contextDir):
@@ -262,7 +263,9 @@ class MakeAppWorkDir:
 		else:
 			self.msg("\t%s" % contextDir)
 			os.makedirs(contextDir)
-		for name in exampleContext:
+		contextNames = exampleContext.keys()
+		contextNames.sort()
+		for name in contextNames:
 			filename = os.path.join(contextDir, name)
 			if os.path.exists(filename):
 				self.msg("\t%s already exists." % filename)
@@ -301,7 +304,8 @@ class MakeAppWorkDir:
 			'ErrorMsgs': '[a-zA-Z0-9]*',
 			'Logs': '[a-zA-Z0-9]*',
 			'Sessions': '[a-zA-Z0-9]*',
-			self._contextName: '*.pyc\n*.pyo' }
+			self._contextName: '*.pyc\n*.pyo'
+		}
 		foundFiles = 0
 		for dir, contents in files.items():
 			filename = os.path.join(self._workDir,
@@ -322,7 +326,7 @@ class MakeAppWorkDir:
 		self.msg("Changing the ownership...")
 		uid = self._uid
 		if uid is None:
-			uid =os.getuid()
+			uid = os.getuid()
 		gid = self._gid
 		if gid is None:
 			gid = os.getgid()
@@ -479,15 +483,15 @@ class Main(Page):
 		which you can get to from here:</p>
 		<ul>
 		''')
-		adapterName = self.request().adapterName()
+		servletPath = self.request().servletPath()
 		contextName = self.request().contextName()
 		ctxs = self.application().contexts().keys()
 		ctxs.sort()
 		for ctx in ctxs:
-			if ctx in ('default', contextName) or ctx.find('/') >= 0:
+			if ctx in ('default', contextName) or '/' in ctx:
 				continue
 			self.writeln('<li><a href="%s/%s/">%s</a></li>'
-				% (adapterName, ctx, ctx))
+				% (servletPath, ctx, ctx))
 		self.writeln('</ul>')
 """
 
@@ -551,7 +555,7 @@ def main(args=None):
 				print 'Error: Group %r does not exist.' % gid
 				sys.exit(2)
 			except ImportError:
-				print 'Error: Group names are not supported.'
+				print 'Error: Group names are supported under Unix only.'
 				sys.exit(2)
 			gid = entry[2]
 	# Figure out the user id:
@@ -567,7 +571,7 @@ def main(args=None):
 				print 'Error: User %r does not exist.' % uid
 				sys.exit(2)
 			except ImportError:
-				print 'Error: User names are not supported.'
+				print 'Error: User names are supported under Unix only.'
 				sys.exit(2)
 			if not gid:
 				gid = entry[3]
