@@ -1,35 +1,36 @@
 #!/usr/bin/env python
 
-"""
-install.py
-Webware for Python
+"""install.py
+
+Webware for Python installer
 
 FUTURE
 	* Look for an install.py in each component directory and run it
 	  (there's not a strong need right now).
+	* Use distutils or setuptools instead of our own plugin concept.
+
 """
 
 
 import os, sys
 from glob import glob
-from MiscUtils.PropertiesObject import PropertiesObject
 
-try:
-	from cStringIO import StringIO
-except ImportError:
-	from StringIO import StringIO
+from MiscUtils import StringIO
+from MiscUtils.PropertiesObject import PropertiesObject
 
 
 class OutputCatcher:
 	"""Auxiliary class for logging output."""
+
 	def __init__(self, output, log):
-		self.output = output
-		self.log = log
+		self._output = output
+		self._log = log
+
 	def write(self, stuff):
 		if stuff:
-			self.output.write(stuff)
-			self.output.flush()
-			self.log.append(stuff)
+			self._output.write(stuff)
+			self._output.flush()
+			self._log.append(stuff)
 
 
 class Installer:
@@ -54,10 +55,13 @@ class Installer:
 		self._autotoc = AutoToC()
 
 
-	## debug printing facility ##
+	## Debug printing facility ##
 
-	def _nop (self, msg): pass
-	def _printMsg (self, msg): print '  ' + msg
+	def _nop (self, msg):
+		pass
+
+	def _printMsg (self, msg):
+		print '  ' + msg
 
 
 	## Running the installation ##
@@ -99,35 +103,28 @@ class Installer:
 
 	def printHello(self):
 		from time import time, localtime, asctime
-		from string import replace
 		print '%(name)s %(versionString)s' % self._props
 		print 'Installer'
 		print
 		self.printKeyValue('Cur Date', asctime(localtime(time())))
-		self.printKeyValue('Python', replace(sys.version, ') [', ')\n['))
+		self.printKeyValue('Python', sys.version.replace(') [', ')\n['))
 		self.printKeyValue('Op Sys', os.name)
 		self.printKeyValue('Platform', sys.platform)
 		self.printKeyValue('Cur Dir', os.getcwd())
 		print
 
-	def checkPyVersion(self, minver=(2,0)):
+	def checkPyVersion(self, minver=(2, 0)):
 		"""Check for minimum required Python version."""
 		try:
 			ver = sys.version_info[:len(minver)]
-			version = '.'.join(map(str, ver))
-			minversion = '.'.join(map(str, minver))
-		except AttributeError: # Python < 2.0
-			from string import split, join
-			ver = tuple(map(int, split(split(sys.version, ' ', 1)[0], '.')[:len(minver)]))
-			version = join(map(str, ver), '.')
-			minversion = join(map(str, minver), '.')
+		except AttributeError:
+			ver = (1,)
 		if ver < minver:
-			print 'This Release of Webware requires Python %s.' % minversion
-			print 'Your currently used version is Python %s.' % version
-			print 'Please go to http://www.python.org for the latest version of Python.'
-			if ver[0] <= 1: # require at least Python 2.0 for installation
-				return 0 # otherwise stop here
-			response = raw_input('\nYou may continue to install, '
+			print ('This Release of Webware requires Python %s.\n'
+				'Your currently used version is Python %s.\n'
+				'You can download a newer version at: http://www.python.org\n'
+				% ('.'.join(map(str, minver)), '.'.join(map(str, ver))))
+			response = raw_input('You may continue to install, '
 				'but Webware may not perform as expected.\n'
 				'Do you wish to continue with the installation?  [yes/no] ')
 			return response[:1].upper() == "Y"
@@ -137,44 +134,53 @@ class Installer:
 		try:
 			import threading
 		except ImportError:
-			print '!!! Webware requires that Python be compiled with threading support.'
-			print 'This version of Python does not appear to support threading.'
-			response = raw_input('\nYou may continue, '
+			print ('Webware requires that Python be compiled with threading support.\n'
+				'This version of Python does not appear to support threading.\n')
+			response = raw_input('You may continue, '
 				'but you will have to run the AppServer with a Python\n'
-				'interpreter that has threading enabled.'
+				'interpreter that has threading enabled.\n'
 				'Do you wish to continue with the installation? [yes/no] ')
 			return response[:1].upper() == "Y"
 		return 1
 
 	def detectComponents(self):
 		print 'Scanning for components...'
-		dirnames = filter(lambda dir: not dir.startswith('.')
+		dirNames = filter(lambda dir: not dir.startswith('.')
 				and os.path.isdir(dir), os.listdir(os.curdir))
-		maxLen = max(map(len, dirnames))
+		dirNames.sort()
+		self._maxCompLen = max(map(len, dirNames))
+		oldPyVersion = 0
 		column = 0
-		for dirname in dirnames:
-			propName = os.path.join(dirname, 'Properties.py')
+		for dirName in dirNames:
+			propName = dirName + '/Properties.py'
 			try:
-				print dirname.ljust(maxLen, '.'),
+				print dirName.ljust(self._maxCompLen, '.'),
 			except TypeError:
-				print dirname.ljust(maxLen),
+				print dirName.ljust(self._maxCompLen),
 			if os.path.exists(propName):
 				comp = PropertiesObject(propName)
-				comp['dirname'] = dirname
-				if not comp.has_key('releaseDate'):
-					comp['releaseDate'] = self._props['releaseDate']
-				self._comps.append(comp)
-				print 'yes',
+				comp['dirname'] = dirName
+				for key in self._props.keys():
+					if not comp.has_key(key):
+						comp[key] = self._props[key]
+				if sys.version_info[:3] < comp['requiredPyVersion']:
+					oldPyVersion += 1
+					print 'no*',
+				else:
+					self._comps.append(comp)
+					print 'yes',
 			else:
 				print 'no ',
 			if column < 2 and not self._verbose:
 				print '   ',
-				column = column + 1
+				column += 1
 			else:
 				print
 				column = 0
 		if column:
 			print
+		if oldPyVersion:
+			print "* some components require a newer Python version"
 		self._comps.sort(lambda a, b: cmp(a['name'], b['name']))
 		print
 
@@ -189,8 +195,8 @@ class Installer:
 				print 'a password will be automatically generated.'
 			else:
 				print 'the password specified on the command-line will be used.'
-			import getpass
-			password = getpass.getpass()
+			from getpass import getpass
+			password = getpass()
 		else:
 			if defpass is None:
 				print 'A password will be automatically generated.'
@@ -198,7 +204,8 @@ class Installer:
 				print 'A password was specified on the command-line.'
 			password = None
 		print 'You can check the password after installation at:'
-		print 'WebKit/Configs/Application.config'
+		appConfig = 'WebKit/Configs/Application.config'
+		print os.path.normpath(appConfig)
 		if not password:
 			if defpass is None:
 				from string import letters, digits
@@ -207,7 +214,7 @@ class Installer:
 			else:
 				password = defpass
 		try: # read config file
-			data = open('WebKit/Configs/Application.config').read()
+			data = open(appConfig).read()
 		except IOError:
 			print 'Error reading Application.config file.'
 			print 'Password not replaced, make sure to edit it by hand.'
@@ -231,7 +238,7 @@ class Installer:
 				print "'AdminPassword' not found in config file."
 			return
 		try: # write back config file
-			open('WebKit/Configs/Application.config', 'w').write(data)
+			open(appConfig, 'w').write(data)
 		except IOError:
 			print 'Error writing Application.config (probably no permission).'
 			print 'Password not replaced, make sure to edit it by hand.'
@@ -274,9 +281,8 @@ class Installer:
 		print
 
 	def createBrowsableSource(self):
-		"""Create HTML documents for class hierarchies, summaries, source files, etc."""
+		"""Create HTML docs for class hierarchies, summaries, sources etc."""
 		print 'Creating html source, summaries and doc files...'
-		maxLen = max(map(lambda comp: len(comp['dirname']), self._comps))
 		column = 0
 		for comp in self._comps:
 			dir = comp['dirname']
@@ -284,10 +290,10 @@ class Installer:
 				print dir, '...'
 			else:
 				try:
-					print dir.ljust(maxLen, '.'),
+					print dir.ljust(self._maxCompLen, '.'),
 				except TypeError:
-					print dir.ljust(maxLen),
-			sourceDir = '%s/Docs/Source' % dir
+					print dir.ljust(self._maxCompLen),
+			sourceDir = dir + '/Docs/Source'
 			self.makeDir(sourceDir)
 			filesDir = sourceDir + '/Files'
 			self.makeDir(filesDir)
@@ -295,7 +301,7 @@ class Installer:
 			self.makeDir(summariesDir)
 			docsDir = sourceDir + '/Docs'
 			self.makeDir(docsDir)
-			for pyFilename in glob('%s/*.py' % dir):
+			for pyFilename in glob(dir + '/*.py'):
 				self.createHighlightedSource(pyFilename, filesDir)
 				self.createPySummary(pyFilename, summariesDir)
 				self.createPyDocs(pyFilename, docsDir)
@@ -305,8 +311,8 @@ class Installer:
 			if not self._verbose:
 				print "ok",
 				if column < 2:
-					print '   ',
-					column = column + 1
+					print '    ',
+					column += 1
 				else:
 					print
 					column = 0
@@ -361,7 +367,7 @@ class Installer:
 			sys.stdout = StringIO()
 			try:
 				pydoc.writedoc(module)
-			except:
+			except Exception:
 				pass
 			msg = sys.stdout.getvalue()
 			sys.stdout = stdout
@@ -456,8 +462,7 @@ class Installer:
 			comp['htDocs'] = ht
 			# Set up release notes
 			ht = []
-			files = glob(os.path.join(comp['dirname'],
-				'Docs', 'RelNotes-*.html'))
+			files = glob(comp['dirname'] + '/Docs/RelNotes-*.html')
 			if files:
 				releaseNotes = []
 				for filename in files:
@@ -483,7 +488,7 @@ class Installer:
 			ht = '\n'.join(ht)
 			comp['htReleaseNotes'] = ht
 			# Write file
-			filename = os.path.join(comp['dirname'], 'Docs', 'index.html')
+			filename = comp['dirname'] + '/Docs/index.html'
 			ht = self.processPyTemplate(index, comp)
 			open(filename, 'w').write(ht)
 		if not keep:
@@ -493,47 +498,29 @@ class Installer:
 	def createDocContexts(self):
 		"""Create a WebKit context for every Docs directory."""
 		print 'Making all Docs directories browsable via WebKit...'
+		# Place an __init__.py file in every Docs directory
 		docsDirs = ['Docs']
 		for comp in self._comps:
-			docsDirs.append(comp['dirname'] + '/Docs')
-		config = []
+			if comp.get('docs', None):
+				docsDir = comp['dirname'] + '/Docs'
+				if os.path.isdir(docsDir):
+					docsDirs.append(docsDir)
 		for docsDir in docsDirs:
-			if os.path.exists(docsDir):
-				open(docsDir + '/__init__.py', 'w').write(
-					'# Allows this directory to be used as a WebKit context.\n')
-			config.append("Contexts['%s'] = WebwarePath + '/%s'" % ((docsDir,)*2))
-		config = '\n'.join(config)
-		try: # read config file
-			data = open('WebKit/Configs/Application.config').read()
-		except IOError:
-			print 'Error reading Application.config file.'
-			print 'Docs cannot be made browsable via WebKit.'
-			data = ''
-		if data.find(config) < 0:
-			insertMark = "# Installer will insert Contexts['Docs'] here."
-			insertPos = data.find(insertMark)
-			if insertPos < 0:
-				print 'Configuration has already been changed.'
-				print 'Docs directories will not be added as context.'
-			else:
-				data = data[:insertPos] + config + data[insertPos+len(insertMark):]
-			try: # write back config file
-				open('WebKit/Configs/Application.config', 'w').write(data)
-			except IOError:
-				print 'Error writing Application.config (probably no permission).'
-				print 'Docs cannot be made browsable via WebKit.'
-		else:
-			print 'Docs directories are already registered with WebKit.'
-		# Copy favicon to the default context:
+			initFile = docsDir + '/__init__.py'
+			if not os.path.exists(initFile) or 1:
+				open(initFile, 'w').write(
+					'# this can be browsed as a Webware context\n')
+		# Copy favicon to the default context
 		open('WebKit/Examples/favicon.ico', 'wb').write(
 			open('Docs/favicon.ico', 'rb').read())
 		print
 
 	def backupConfigs(self):
-		"""Copy *.config to *.config.default, if the .default files don't already exist.
+		"""Copy *.config to *.config.default, if they don't already exist.
+
+		This allows the user to always go back to the default config file if
+		needed (for troubleshooting for example).
 
-		This allows the user to always go back to the default config file if needed
-		(for troubleshooting for example).
 		"""
 		print 'Creating backups of original config files...'
 		self._backupConfigs(os.curdir)
@@ -576,16 +563,19 @@ class Installer:
 			open(t, 'wb').write(open(s, 'rb').read())
 			print
 
-	def compileModules(self):
-		import compileall
+	def compileModules(self, force=0):
+		"""Compile modules in all installed componentes."""
+		from compileall import compile_dir
 		print 'Byte compiling all modules...'
-		try:
-			compileall.compile_dir(os.curdir, 10, None, 1, None, 1)
-		except TypeError: # workaround for Python < 2.3
-			stdout = sys.stdout
-			sys.stdout = StringIO()
-			compileall.compile_dir(os.curdir, 10, None, 1)
-			sys.stdout = stdout
+		for comp in self._comps:
+			dir = comp['dirname']
+			try:
+				compile_dir(dir, force=force, quiet=1)
+			except TypeError: # workaround for Python < 2.3
+				stdout = sys.stdout
+				sys.stdout = StringIO()
+				compile_dir(dir, force=force)
+				sys.stdout = stdout
 		print
 
 	def fixPermissions(self):
@@ -671,6 +661,7 @@ Installation is finished.''' % ((os.sep,)*2)
 	def processPyTemplate(self, input, props):
 		"""Process a Python template."""
 		global scope
+
 		def header(title, titleclass=None, style=None):
 			"""Get the header of a document."""
 			if not titleclass:
@@ -692,9 +683,11 @@ Installation is finished.''' % ((os.sep,)*2)
 					'<!--', style, '-->', '</style>'))
 			css = '\n'.join(css)
 			return scope['htHeader'] % locals()
+
 		def footer():
 			"""Get the footer of a document."""
 			return scope['htFooter']
+
 		scope = props.copy()
 		try:
 			scope = dict(props)
@@ -708,14 +701,15 @@ Installation is finished.''' % ((os.sep,)*2)
 
 
 def printHelp():
-	print 'Usage: install.py [options]'
-	print 'Install WebWare in the local directory.'
+	print "Usage: install.py [options]"
+	print "Install WebWare in the local directory."
 	print
-	print '  -h, --help            Print this help screen.'
-	print '  -v, --verbose         Print extra information messages during install.'
-	print '  --no-password-prompt  Do not prompt for the WebKit password during install.'
-	print '  --set-password=...    Set the WebKit password to the given value.'
-	print '  --keep-templates      Keep the templates for creating the docs.'
+	print "  -h, --help            Print this help screen."
+	print "  -v, --verbose         Print extra information messages during install."
+	print "  --no-password-prompt  Don't prompt for WebKit password during install."
+	print "  --set-password=...    Set the WebKit password to the given value."
+	print "  --keep-templates      Keep the templates for creating the docs."
+
 
 if __name__ == '__main__':
 	import getopt
@@ -729,7 +723,7 @@ if __name__ == '__main__':
 	else:
 		for o, a in opts:
 			if o in ("-v", "--verbose"):
-				verbose=1
+				verbose = 1
 			elif o == "--no-password-prompt":
 				passprompt = 0
 			elif o == "--set-password":
